@@ -90,9 +90,11 @@ decl_blend_dir_fn(dav1d_blend_h_ssse3);
 decl_warp8x8_fn(dav1d_warp_affine_8x8_avx2);
 decl_warp8x8_fn(dav1d_warp_affine_8x8_sse4);
 decl_warp8x8_fn(dav1d_warp_affine_8x8_ssse3);
+decl_warp8x8_fn(dav1d_warp_affine_8x8_sse2);
 decl_warp8x8t_fn(dav1d_warp_affine_8x8t_avx2);
 decl_warp8x8t_fn(dav1d_warp_affine_8x8t_sse4);
 decl_warp8x8t_fn(dav1d_warp_affine_8x8t_ssse3);
+decl_warp8x8t_fn(dav1d_warp_affine_8x8t_sse2);
 
 decl_emu_edge_fn(dav1d_emu_edge_avx2);
 decl_emu_edge_fn(dav1d_emu_edge_ssse3);
@@ -104,6 +106,13 @@ COLD void bitfn(dav1d_mc_dsp_init_x86)(Dav1dMCDSPContext *const c) {
     c->mct[type] = dav1d_prep_##name##_##suffix
     const unsigned flags = dav1d_get_cpu_flags();
 
+    if(!(flags & DAV1D_X86_CPU_FLAG_SSE2))
+        return;
+
+#if BITDEPTH == 8
+    c->warp8x8  = dav1d_warp_affine_8x8_sse2;
+    c->warp8x8t = dav1d_warp_affine_8x8t_sse2;
+#endif
 
     if(!(flags & DAV1D_X86_CPU_FLAG_SSSE3))
         return;
