@@ -27,12 +27,13 @@
 
 #include "config.h"
 
-#include <assert.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "common/attributes.h"
+
 #include "input/input.h"
 #include "input/demuxer.h"
 
